@@ -29,7 +29,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "faust/misc.h"
 #include "faust/dsp/dsp.h"
 #include "faust/audio/audio.h"
 #include "faust/gui/meta.h"
@@ -38,7 +37,6 @@
 #include "faust/gui/MidiUI.h"
 #include "faust/dsp/poly-dsp.h"
 #include "faust/dsp/faust-engine.h"
-#include "faust/dsp/dsp-combiner.h"
 
 //**************************************************************
 // Mono or polyphonic audio DSP engine
@@ -63,16 +61,21 @@ class FaustPolyEngine {
         midi_handler fMidiHandler;
         MidiUI fMidiUI;
     
+        bool checkPolyphony(dsp* mono_dsp)
+        {
+            if (fJSONUI.find("keyboard") != std::string::npos) return true;
+            if (fJSONUI.find("poly") != std::string::npos) return true;
+            return MidiMeta::checkPolyphony(mono_dsp);
+        }
+    
         void init(dsp* mono_dsp, audio* driver, midi_handler* midi)
         {
             bool midi_sync = false;
             int nvoices = 0;
+            fRunning = false;
             
             MidiMeta::analyse(mono_dsp, midi_sync, nvoices);
             
-            fDriver = driver;
-            fRunning = false;
-            
             // Getting the UI JSON
             JSONUI jsonui1(mono_dsp->getNumInputs(), mono_dsp->getNumOutputs());
             mono_dsp->buildUserInterface(&jsonui1);
@@ -83,9 +86,7 @@ class FaustPolyEngine {
             mono_dsp->metadata(&jsonui1M);
             fJSONMeta = jsonui1M.JSON();
             
-            if (fJSONUI.find("keyboard") != std::string::npos
-                || fJSONUI.find("poly") != std::string::npos
-                || nvoices > 0) {
+            if (checkPolyphony(mono_dsp) && (nvoices > 0)) {
                 
                 fPolyDSP = new mydsp_poly(mono_dsp, nvoices, true);
                 fMidiHandler.addMidiIn(fPolyDSP);
@@ -101,6 +102,7 @@ class FaustPolyEngine {
                 JSONUI jsonui2(mono_dsp->getNumInputs(), mono_dsp->getNumOutputs());
                 fFinalDSP->buildUserInterface(&jsonui2);
                 fJSONUI = jsonui2.JSON();
+                
                 JSONUI jsonui2M(mono_dsp->getNumInputs(), mono_dsp->getNumOutputs());
                 fFinalDSP->metadata(&jsonui2M);
                 fJSONMeta = jsonui2M.JSON();
@@ -126,8 +128,19 @@ class FaustPolyEngine {
       
             MyMeta meta;
             fFinalDSP->metadata(&meta);
-            fDriver->init(meta.fName.c_str(), fFinalDSP);
-            if (midi) midi->setName(meta.fName.c_str());
+            if (midi) midi->setName(meta.fName);
+            
+            if (driver) {
+                // If driver cannot be initialized, start will fail later on...
+                if (!driver->init(meta.fName.c_str(), fFinalDSP)) {
+                    delete driver;
+                    fDriver = NULL;
+                } else {
+                    fDriver = driver;
+                }
+            } else {
+                fDriver = NULL;
+            }
         }
     
     public:
@@ -151,11 +164,11 @@ class FaustPolyEngine {
         bool start()
         {
             if (!fRunning) {
-                fRunning = fDriver->start();
+                fRunning = (fDriver) ? fDriver->start() : false;
             }
             return fRunning;
         }
-        
+    
         /*
          * isRunning()
          * Returns true if the DSP frames are being computed and
@@ -174,10 +187,13 @@ class FaustPolyEngine {
         {
             if (fRunning) {
                 fRunning = false;
-                fDriver->stop();
+                if (fDriver) fDriver->stop();
             }
         }
     
+        void setGroup(bool group) { if (fPolyDSP) fPolyDSP->setGroup(group); }
+        bool getGroup() { return (fPolyDSP) ? fPolyDSP->getGroup() : false; }
+    
         /*
          * keyOn(pitch, velocity)
          * Instantiates a new polyphonic voice where velocity
@@ -299,6 +315,11 @@ class FaustPolyEngine {
         {
             fFinalDSP->buildUserInterface(ui_interface);
         }
+    
+        void compute(int count, FAUSTFLOAT** inputs, FAUSTFLOAT** outputs)
+        {
+            fFinalDSP->compute(count, inputs, outputs);
+        }
 
         /*
          * getParamsCount()
@@ -510,7 +531,7 @@ class FaustPolyEngine {
          */
         void setAccConverter(int p, int acc, int curve, float amin, float amid, float amax)
         {
-           fAPIUI.setAccConverter(p, acc, curve, amin, amid, amax);
+            fAPIUI.setAccConverter(p, acc, curve, amin, amid, amax);
         }
 
         /*
@@ -536,7 +557,7 @@ class FaustPolyEngine {
          * getCPULoad()
          * Return DSP CPU load.
          */
-        float getCPULoad() { return fDriver->getCPULoad(); }
+        float getCPULoad() { return (fDriver) ? fDriver->getCPULoad() : 0.f; }
 
         /*
          * getScreenColor() -> c:int
